Package: CNVRanger
Version: 1.29.1
Title: Summarization and expression/phenotype association of CNV ranges
Authors@R: c(
    person("Ludwig", "Geistlinger", email = "ludwig.geistlinger@gmail.com",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-2495-5464")),
    person("Vinicius Henrique", "da Silva", role = "aut"),
    person("Marcel", "Ramos",
    role = "ctb", comment = c(ORCID = "0000-0002-3242-0582")),
    person("Levi", "Waldron",
    role = "ctb", comment = c(ORCID = "0000-0003-2725-0694"))
  )
Depends: GenomicRanges, RaggedExperiment
Imports: BiocGenerics, BiocParallel, GDSArray, GenomeInfoDb, IRanges,
        S4Vectors, SNPRelate, SummarizedExperiment, data.table, edgeR,
        gdsfmt, grDevices, lattice, limma, methods, plyr, qqman,
        rappdirs, reshape2, stats, utils
Suggests: AnnotationHub, BSgenome.Btaurus.UCSC.bosTau6.masked,
        BiocStyle, ComplexHeatmap, Gviz, MultiAssayExperiment,
        TCGAutils, TxDb.Hsapiens.UCSC.hg19.knownGene, curatedTCGAData,
        ensembldb, grid, knitr, org.Hs.eg.db, regioneR, rmarkdown,
        statmod
Description: The CNVRanger package implements a comprehensive tool suite for CNV analysis.
    This includes functionality for summarizing individual CNV calls across a population,
    assessing overlap with functional genomic regions, and association analysis 
    with gene expression and quantitative phenotypes.
License: Artistic-2.0
BugReports: https://github.com/waldronlab/CNVRanger/issues
Encoding: UTF-8
VignetteBuilder: knitr
biocViews: CopyNumberVariation, DifferentialExpression, GeneExpression,
        GenomeWideAssociation, GenomicVariation, Microarray, RNASeq,
        SNP
RoxygenNote: 7.2.3
Date: 2025-05-08
git_url: https://git.bioconductor.org/packages/CNVRanger
git_branch: devel
git_last_commit: 53d628f
git_last_commit_date: 2026-04-29
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 21:39:54 UTC; biocbuild
Author: Ludwig Geistlinger [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2495-5464>),
  Vinicius Henrique da Silva [aut],
  Marcel Ramos [ctb] (ORCID: <https://orcid.org/0000-0002-3242-0582>),
  Levi Waldron [ctb] (ORCID: <https://orcid.org/0000-0003-2725-0694>)
Maintainer: Ludwig Geistlinger <ludwig.geistlinger@gmail.com>
Built: R 4.6.0; ; 2026-05-19 04:46:28 UTC; unix
