# Generated by roxygen2: do not edit by hand

export(cnvEQTL)
export(cnvGWAS)
export(cnvOncoPrint)
export(generateGDS)
export(importLrrBaf)
export(plotEQTL)
export(plotManhattan)
export(plotRecurrentRegions)
export(populationRanges)
export(setupCnvGWAS)
import(GenomeInfoDb)
import(GenomicRanges)
import(RaggedExperiment)
import(S4Vectors)
importFrom(IRanges,extractList)
importFrom(IRanges,subsetByOverlaps)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,pdf)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,ecdf)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,pchisq)
importFrom(stats,qbeta)
importFrom(stats,qchisq)
importFrom(utils,download.file)
importFrom(utils,read.table)
importFrom(utils,unzip)
importFrom(utils,write.table)
