# CNVRanger <a href='https://bioconductor.org/packages/CNVRanger'><img src='https://raw.githubusercontent.com/Bioconductor/BiocStickers/devel/CNVRanger/CNVRanger.png' align="right" height="139" /></a>
Summarization and expression/phenotype association analysis for CNV ranges

Bioinformatics publication: https://doi.org/10.1093/bioinformatics/btz632  
Bioconductor package: https://bioconductor.org/packages/CNVRanger  
CNV workshop: https://github.com/waldronlab/CNVWorkshop

The [CNVRanger](https://bioconductor.org/packages/CNVRanger) package implements
a comprehensive tool suite for CNV analysis. This includes functionality for
summarizing individual CNV calls across a population, assessing overlap with
functional genomic regions, and association analysis with gene expression and
quantitative phenotypes.
