#' @import S4Vectors GenomeInfoDb GenomicRanges RaggedExperiment
#' @importFrom IRanges extractList subsetByOverlaps
#' @importFrom grDevices dev.off pdf extendrange
#' @importFrom methods as is
#' @importFrom stats ecdf lm median pchisq qbeta qchisq
#' @importFrom utils download.file read.table unzip write.table
NULL
